/*
   Use this program to test your development environment
   and the Graphviz program.

   Also use this program as a hint for SearchForInteger.java.
*/

import binarytree.*;

import java.util.List;
import java.util.ArrayList;

/**
   A program that tests the findEven() and findOdd() methods.
*/
public class SearchForEvenOdd_soln
{
   /**
      Given a binary tree of integers, return a
      list of all the even integers from the tree.
   */
   public static List<Integer> findEven(BTree<Integer> btree)
   {
      return findEvenOdd(btree, 0);
   }

   /**
      Given a binary tree of integers, return a
      list of all the odd integers from the tree.
   */
   public static List<Integer> findOdd(BTree<Integer> btree)
   {
      return findEvenOdd(btree, 1);
   }


   /**
      Given a binary tree of integers, return a list of
      all the even or odd integers from the tree depending
      on the value of the zero_or_one parameter, 0 for
      even integers and 1 for odd integers.
   */
   private static List<Integer> findEvenOdd(BTree<Integer> btree,
                                            int zero_or_one)
   {
      List<Integer> result = new ArrayList<>();
      if ( btree.isEmpty() )
      {
         return result;
      }
      else if ( btree.isLeaf() )
      {
         final int element = btree.root();
         if ( zero_or_one == element % 2 ) // even or odd
         {
            result.add( element );
         }
         return result;
      }
      else
      {
         List<Integer> leftResult = findEvenOdd(btree.left(), zero_or_one);
         List<Integer> rightResult = findEvenOdd(btree.right(), zero_or_one);
         final int element = btree.root();
         if ( zero_or_one == element % 2 ) // even or odd
         {
            result.add( element );
         }
         result.addAll( leftResult );
         result.addAll( rightResult );
         return result;
      }
   }


   // Test the findOdd() and findEven() methods.
   public static void main(String[] args)
   {
      BTree<Integer> btree =
           new BTreeLinked<>(0,
               new BTreeLinked<>(0,
                   new BTreeLinked<>(0,
                       new BTreeLinked<>(0,
                           new BTreeLinked<>(0),
                           new BTreeLinked<>(0)),
                       new BTreeLinked<>(1,
                           new BTreeLinked<>(0),
                           new BTreeLinked<>(0))),
                   new BTreeLinked<>(0,
                       new BTreeLinked<>(0),
                       new BTreeLinked<>(0))),
               new BTreeLinked<>(0,
                   new BTreeLinked<>(0),
                   new BTreeLinked<>(0,
                       new BTreeLinked<>(0,
                           new BTreeLinked<>(0,
                               new BTreeLinked<>(0,
                                   new BTreeLinked<>(0),
                                   new BTreeLinked<>(0)),
                               new BTreeLinked<>(3,
                                   new BTreeLinked<>(0),
                                   new BTreeLinked<>(0))),
                           new BTreeLinked<>(0,
                               new BTreeLinked<>(0,
                                   new BTreeLinked<>(0),
                                   new BTreeLinked<>(0)),
                               new BTreeLinked<>(0,
                                   new BTreeLinked<>(0),
                                   new BTreeLinked<>(5)))),
                       new BTreeLinked<>(0))));

      System.out.println( btree );
      BTree2dot.btree2dot(btree, "btree");
      BTree2png.btree2png("btree");
      List<Integer> odds = findOdd( btree );
      System.out.println( odds );
      List<Integer> evens = findEven( btree );
      System.out.println( evens );
   }
}
